==> cego-2.6.2.tar.gz: cego/src/CegoIndexEntry.cc <==
///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoIndexEntry.cc
// -----------------
// Index entry class implementation
//     
// Design and Implementation by Bjoern Lemke               
//     
// (C)opyright 2000-2006 Bjoern Lemke
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: CegoIndexEntry
// 
// Description: Implementation of tree entry for full balanced index trees
//
// Status: CLEAN
//
///////////////////////////////////////////////////////////////////////////////

#include "Exception.h"
#include "CegoIndexEntry.h"

CegoIndexEntry::CegoIndexEntry()
{
    _pI = 0;
    _len = 0;
    _isLocal = false;
    _tid = 0;
}

CegoIndexEntry::CegoIndexEntry(int tid)
{
    _pI = 0;
    _len = 0;
    _isLocal = false;
    _tid = tid;
}

CegoIndexEntry::~CegoIndexEntry()
{
    if (_isLocal)
    {
	free (_pI);
    }
}

void CegoIndexEntry::initEntry(const CegoDataPointer& dp, char* idxPtr, int idxLen)
{
    
    if (_isLocal)
	free (_pI);

    CegoDataPointer zp(0,0,0);

    _len = 4 * zp.getEncodingLength() + sizeof(char) + sizeof(int) + idxLen;

    _pI = malloc(_len);

    if (_pI == 0 )
    {
	throw Exception(EXLOC, "Cannot allocate index entry");
    }

    _isLocal = true;


    setParent(zp);
    setLeftBranch(zp);
    setRightBranch(zp);
    setData(dp);
    setHeight(0);
    setTID(_tid);
    
    _idxDataPtr =  (char*)((long)_pI + _len - idxLen);
    
    memcpy(_idxDataPtr, idxPtr, idxLen);
    
}

void CegoIndexEntry::setPtr(void* p, int len)
{
    _isLocal = false;
    _pI = p;
    _len = len;

    CegoDataPointer dp;
    _idxDataPtr = (char*)((long)_pI + 4 * dp.getEncodingLength() + sizeof(char) + sizeof(int));
    
}

void* CegoIndexEntry::getPtr()
{
    return _pI;
}

int CegoIndexEntry::getLen() const
{
    return _len;
}
   
void CegoIndexEntry::setParent(const CegoDataPointer& dp)
{

    // int* p = (int*)_pI;

    dp.encode(_pI);

    /*
    *p = dp.getFileId();
    p++;
    *p = dp.getPageId();
    p++;
    *p = dp.getOffset();
    */
}

CegoDataPointer CegoIndexEntry::getParent() const
{
    
    CegoDataPointer dp;

    dp.decode(_pI);

    /*
      CegoDataPointer dp(
      *((int*)_pI),
      *((int*)((long)_pI + sizeof(int))),
      *((int*)((long)_pI + 2 * sizeof(int))));
      
      */
    return dp;

}

void CegoIndexEntry::setLeftBranch(const CegoDataPointer& dp)
{


    dp.encode((void*) ((long)_pI + (long)dp.getEncodingLength()));
    
    /*
    int* p = (int*)((long)_pI + 3 * sizeof(int));
    
    *p = dp.getFileId();
    p++;
    *p = dp.getPageId();
    p++;
    *p = dp.getOffset();
    */
    
}

CegoDataPointer CegoIndexEntry::getLeftBranch() const
{

    CegoDataPointer dp;

    dp.decode((void*) ((long)_pI + (long)dp.getEncodingLength()));

    /*

      CegoDataPointer dp(
      *((int*)((long)_pI + 3 * sizeof(int))),
      *((int*)((long)_pI + 4 * sizeof(int))),
      *((int*)((long)_pI + 5 * sizeof(int))));
      
      */

    return dp;
    
}

void CegoIndexEntry::setRightBranch(const CegoDataPointer& dp)
{


    dp.encode((void*) ((long)_pI + 2 * (long)dp.getEncodingLength()));
    
    /* 
       int* p = (int*)((long)_pI + 6 * sizeof(int));
       
       *p = dp.getFileId();
       p++;
       *p = dp.getPageId();
       p++;
       *p = dp.getOffset();
       */
 
}

CegoDataPointer CegoIndexEntry::getRightBranch() const
{


    CegoDataPointer dp;

    dp.decode((void*) ((long)_pI + 2 * (long)dp.getEncodingLength()));

    /*
      CegoDataPointer dp(
      *((int*)((long)_pI + 6 * sizeof(int))),
      *((int*)((long)_pI + 7 * sizeof(int))),
      *((int*)((long)_pI + 8 * sizeof(int))));
      
      */
    
    return dp;
}


void CegoIndexEntry::setData(const CegoDataPointer& dp)
{


    dp.encode((void*) ((long)_pI + 3 * (long)dp.getEncodingLength()));

    /*
      int* p = (int*)((long)_pI + 9 * sizeof(int));
      
      *p = dp.getFileId();
      p++;
      *p = dp.getPageId();
      p++;
      *p = dp.getOffset();
      */

}

CegoDataPointer CegoIndexEntry::getData() const
{


    CegoDataPointer dp;

    dp.decode((void*) ((long)_pI + 3 * (long)dp.getEncodingLength()));

    /*
      
    CegoDataPointer dp(
	*((int*)((long)_pI + 9 * sizeof(int))),
	*((int*)((long)_pI + 10 * sizeof(int))),
	*((int*)((long)_pI + 11 * sizeof(int))));
	*/
    return dp;

}

char* CegoIndexEntry::getIdxPtr()
{
    return _idxDataPtr;
}

int CegoIndexEntry::getIdxLen()
{
    CegoDataPointer dp;
    return _len - 4 * dp.getEncodingLength() + sizeof(char) + sizeof(int);
}


void CegoIndexEntry::setHeight(char h)
{   
    CegoDataPointer dp;
    char* p = (char*)((long)_pI +  4 * dp.getEncodingLength() );
    *p = h;
}

char CegoIndexEntry::getHeight() const
{
    CegoDataPointer dp;
    char h = *((char*)((long)_pI +  4 * dp.getEncodingLength() ));
    return (h);
}

void CegoIndexEntry::setTID(int tid)
{

    CegoDataPointer dp;
    void* pTid = (int*)((long)_pI + 4 * dp.getEncodingLength() + sizeof(char));
    memcpy(pTid, &tid, sizeof(int));
}

int CegoIndexEntry::getTID() const
{
    CegoDataPointer dp;
    void* pTid = (void*)((long)_pI + 4 * dp.getEncodingLength() + sizeof(char));
    int tid;
    memcpy(&tid, pTid, sizeof(int));
    return (tid);
}


CegoIndexEntry& CegoIndexEntry::operator = (const CegoIndexEntry& ie)
{
    _pI = ie._pI;
    _len = ie._len;
    _idxDataPtr = ie._idxDataPtr;
    return (*this);
}

==> cego-2.6.8.tar.gz: cego/src/CegoIndexEntry.cc <==
///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoIndexEntry.cc
// -----------------
// Cego index entry class implementation
//     
// Design and Implementation by Bjoern Lemke               
//     
// (C)opyright 2000-2010 Bjoern Lemke
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// IMPLEMENTATION MODULE
//
// Class: CegoIndexEntry
// 
// Description: Implementation of tree entry for full balanced index trees
//
// Status: QG-2.6
//
///////////////////////////////////////////////////////////////////////////////

// base includes
#include "Exception.h"

// cego includes
#include "CegoIndexEntry.h"

CegoIndexEntry::CegoIndexEntry()
{
    _pI = 0;
    _len = 0;
    _isLocal = false;
    _tid = 0;
}

CegoIndexEntry::CegoIndexEntry(int tid)
{
    _pI = 0;
    _len = 0;
    _isLocal = false;
    _tid = tid;
}

CegoIndexEntry::~CegoIndexEntry()
{
    if (_isLocal)
    {
	free (_pI);
    }
}

void CegoIndexEntry::initEntry(const CegoDataPointer& dp, char* idxPtr, int idxLen)
{
    
    if (_isLocal)
	free (_pI);

    CegoDataPointer zp(0,0,0);

    _len = 4 * zp.getEncodingLength() + sizeof(char) + sizeof(int) + idxLen;

    _pI = malloc(_len);

    if (_pI == 0 )
    {
	throw Exception(EXLOC, Chain("Cannot allocate index entry"));
    }

    _isLocal = true;

    setParent(zp);
    setLeftBranch(zp);
    setRightBranch(zp);
    setData(dp);
    setHeight(0);
    setTID(_tid);
    
    _idxDataPtr =  (char*)((long)_pI + _len - idxLen);
    memcpy(_idxDataPtr, idxPtr, idxLen);
}

void CegoIndexEntry::setPtr(void* p, int len)
{
    _isLocal = false;
    _pI = p;
    _len = len;
    CegoDataPointer dp;
    _idxDataPtr = (char*)((long)_pI + 4 * dp.getEncodingLength() + sizeof(char) + sizeof(int));
}

void* CegoIndexEntry::getPtr()
{
    return _pI;
}

int CegoIndexEntry::getLen() const
{
    return _len;
}
   
void CegoIndexEntry::setParent(const CegoDataPointer& dp)
{
    dp.encode(_pI);
}

CegoDataPointer CegoIndexEntry::getParent() const
{
    CegoDataPointer dp;
    dp.decode(_pI);
    return dp;
}

void CegoIndexEntry::setLeftBranch(const CegoDataPointer& dp)
{
    dp.encode((void*) ((long)_pI + (long)dp.getEncodingLength()));    
}

CegoDataPointer CegoIndexEntry::getLeftBranch() const
{
    CegoDataPointer dp;
    dp.decode((void*) ((long)_pI + (long)dp.getEncodingLength()));
    return dp;    
}

void CegoIndexEntry::setRightBranch(const CegoDataPointer& dp)
{
    dp.encode((void*) ((long)_pI + 2 * (long)dp.getEncodingLength())); 
}

CegoDataPointer CegoIndexEntry::getRightBranch() const
{
    CegoDataPointer dp;
    dp.decode((void*) ((long)_pI + 2 * (long)dp.getEncodingLength()));
    return dp;
}

void CegoIndexEntry::setData(const CegoDataPointer& dp)
{
    dp.encode((void*) ((long)_pI + 3 * (long)dp.getEncodingLength()));
}

CegoDataPointer CegoIndexEntry::getData() const
{
    CegoDataPointer dp;
    dp.decode((void*) ((long)_pI + 3 * (long)dp.getEncodingLength()));
    return dp;
}

char* CegoIndexEntry::getIdxPtr()
{
    return _idxDataPtr;
}

int CegoIndexEntry::getIdxLen()
{
    CegoDataPointer dp;
    return _len - 4 * dp.getEncodingLength() + sizeof(char) + sizeof(int);
}

void CegoIndexEntry::setHeight(char h)
{   
    CegoDataPointer dp;
    char* p = (char*)((long)_pI +  4 * dp.getEncodingLength() );
    *p = h;
}

char CegoIndexEntry::getHeight() const
{
    CegoDataPointer dp;
    char h = *((char*)((long)_pI +  4 * dp.getEncodingLength() ));
    return (h);
}

void CegoIndexEntry::setTID(int tid)
{
    CegoDataPointer dp;
    void* pTid = (int*)((long)_pI + 4 * dp.getEncodingLength() + sizeof(char));
    memcpy(pTid, &tid, sizeof(int));
}

int CegoIndexEntry::getTID() const
{
    CegoDataPointer dp;
    void* pTid = (void*)((long)_pI + 4 * dp.getEncodingLength() + sizeof(char));
    int tid;
    memcpy(&tid, pTid, sizeof(int));
    return (tid);
}

CegoIndexEntry& CegoIndexEntry::operator = (const CegoIndexEntry& ie)
{
    _pI = ie._pI;
    _len = ie._len;
    _idxDataPtr = ie._idxDataPtr;
    return (*this);
}
